"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestHandler = void 0;
const fetch_1 = require("@sapphire/fetch");
const ratelimits_1 = require("@sapphire/ratelimits");
class RequestHandler {
    constructor(client) {
        this.client = client;
        this.ratelimits = new ratelimits_1.RateLimitManager(1000);
    }
    async request(method, endpoint, body = {}, attempts = 0) {
        const ratelimit = this.ratelimits.acquire(`${method}/${endpoint}`);
        const { baseURL, version } = this.client.options;
        const request = await (0, fetch_1.fetch)(`${baseURL}/${version}/${endpoint}`, {
            method,
            headers: {
                Authorization: this.client.token
            },
            body: JSON.stringify(body)
        }, fetch_1.FetchResultTypes.Result);
        attempts++;
        this.parseRateLimitHeaders(request.headers, ratelimit);
        const retryRequest = () => {
            const retryAfter = request.headers.get('retry-after');
            if (retryAfter) {
                setTimeout(async () => this.request(method, endpoint, body, attempts), Number(retryAfter));
            }
            else {
                void this.request(method, endpoint, body, attempts);
            }
        };
        const { status } = request;
        if (status >= 200 && status < 300) {
            return request.json();
        }
        else if (status === 429) {
            if (attempts >= this.client.options.maxRetries) {
                throw new Error('Too many retries');
            }
            else {
                retryRequest();
            }
        }
        ratelimit.consume();
    }
    parseRateLimitHeaders(headers, ratelimit) {
        const now = Date.now();
        const ratelimitRemaining = headers.get('X-RateLimit-Remaining');
        if (ratelimitRemaining) {
            ratelimit.remaining += Number(ratelimitRemaining) || 0;
        }
        else {
            ratelimit.remaining = 1;
        }
        const retryAfter = headers.get('retry-after');
        const ratelimitReset = headers.get('X-RateLimit-Reset');
        if (retryAfter) {
            ratelimit.expires = (Number(retryAfter) || 1) + now;
        }
        else if (ratelimitReset) {
            ratelimit.expires = Math.max(Number(ratelimitReset), now);
        }
        else {
            ratelimit.expires = now;
        }
    }
}
exports.RequestHandler = RequestHandler;
//# sourceMappingURL=RequestHandler.js.map